/***********************************************************************/
/*  This file is part of the CA ARM C Compiler package                 */
/*  Copyright KEIL ELEKTRONIK GmbH 2002 - 2004                         */
/***********************************************************************/
/*                                                                     */
/*  MAIN.C:  Demonstration of various interrupt functions              */
/*                                                                     */
/***********************************************************************/


#include <LPC21xx.H>    // Philips Peripheral Registers

/*
 * software interrupt function accept parameters and run in
 * supervisor mode (interrupt protected).
 */
int myfunc1 (int i1, long i2) __swi (8)  {
  return (i1 / i2);
}

int myfunc2 (int i1) __swi (9)  {
  return (i1<<4);
}

/*
 * standard interrupt function saves only R0 - R12 and returns with 
 */
void DefaultIRQ (void) __irq {
  ;
}

/*
 * fast interrupt function saves only R0 - R7 and returns with 
 */
void FIQ_Handler (void)	__fiq  {
  IOSET1 = 0x00010000;	        // Set pin P1.16
  EXTINT = 0x00000002;		      // Clear the peripheral interrupt flag
}

/*
 * task functions have no register save/restore and no return.
 */
void tsk (void) __task {
  while (1);
}



/*
 * Sample 'main' function
 */
int res;

void main (void)  {
  IODIR1         = 0x00FF0000;  	// Set the LED pins as outputs
  PINSEL0 	     = 0x20000000;    // Enable the EXTINT1 interrupt
  VICIntSelect   = 0x00008000;	  // Enable a Vic Channel as FIQ
  VICIntEnable   = 0x00008000;		
																	// Set Default interrupt vector
  VICDefVectAddr = (unsigned long) DefaultIRQ;

  res = myfunc1 (10, 2);          // call SWI functions
  res += myfunc2 (res);

	while (1);                      // endless loop
}