/***********************************************************************/
/*  This file is part of the CA ARM C Compiler package                 */
/*  Copyright KEIL ELEKTRONIK GmbH 2002 - 2004                         */
/***********************************************************************/
/*                                                                     */
/*  DebugIO.C:  Debug I/O via Debug Communication Channel (DCC)        */
/*                                                                     */
/***********************************************************************/

#include "DebugIO.H"


#define DBG_PUTCHAR __asm {                 /* Macro: DBG_PUTCHAR      */ \
  __asm         CMP     R0,#0x0A            /* Check 'ch' for LF       */ \
  __asm         BNE     wwait               /* Branch if not LF        */ \
  __asm wwait0: MRC     p14,0,R1,c0,c0      /* Read DCC Control        */ \
  __asm         TST     R1,#0x02            /* Check W bit             */ \
  __asm         BNE     wwait0              /* Branch if W = 1         */ \
  __asm         MOV     R1,#0x0D            /* Insert CR Code          */ \
  __asm         MCR     p14,0,R1,c1,c0      /* Write CR to DCC         */ \
  __asm wwait:  MRC     p14,0,R1,c0,c0      /* Read DCC Control        */ \
  __asm         TST     R1,#0x02            /* Check W bit             */ \
  __asm         BNE     wwait               /* Branch if W = 1         */ \
  __asm         MCR     p14,0,R0,c1,c0      /* Write 'ch' to DCC       */ \
}

#define DBG_GETKEY  __asm {                 /* Macro: DBG_GETKEY       */ \
  __asm rwait:  MRC     p14,0,R0,c0,c0      /* Read DCC Control        */ \
  __asm         TST     R0,#0x01            /* Check R bit             */ \
  __asm         BEQ     rwait               /* Branch if R = 0         */ \
  __asm         MRC     p14,0,R0,c1,c0      /* Read from DCC           */ \
}

#define DBG_ISKEY   __asm {                 /* Macro: DBG_ISKEY        */ \
  __asm rwait:  MRC     p14,0,R0,c0,c0      /* Read DCC Control        */ \
  __asm         AND     R0,R0,#0x01         /* Mask R bit              */ \
}

#define DBG_CR2LF   __asm {                 /* Macro: DBG_CR2LF        */ \
  __asm         CMP     R0,#0x0D            /* Check for CR            */ \
  __asm         MOVEQ   R0,#0x0A            /* Replace CR with LF      */ \
}


// Write Character to Debug Channel
int __dbg_putchar (int ch) __arm {
  DBG_PUTCHAR
}

// Read Character with echo from Debug Channel
int __dbg_getchar (void) __arm {
  DBG_GETKEY
  DBG_CR2LF
  DBG_PUTCHAR
}

// Read Character from Debug Channel
int __dbg_getkey (void) __arm {
  DBG_GETKEY
}

// Check if Character is available
int __dgb_iskey (void) __arm {
  DBG_ISKEY
}
