/******************************************************************************/
/*  This file is part of the CA ARM C Compiler package                        */
/*  Copyright KEIL ELEKTRONIK GmbH 2002 - 2004                                */
/******************************************************************************/
/*                                                                            */
/*  IRQ.S:  IRQ Handler                                                       */
/*                                                                            */
/******************************************************************************/


// Enhanced Interrupt Controller (EIC) definitions
        EIC_BASE        EQU     0xFFFFF800  /* EIC Base Address */
        CICR_OFS        EQU     0x04        /* Curr. Int. Channel R. Offset */
        IVR_OFS         EQU     0x18        /* Interrupt Vector Reg. Offset */
        IPR_OFS         EQU     0x40        /* Interrupt Pending Reg. Offset */


AREA ?C?IRQ, CODE, READONLY, ALIGN=2

PUBLIC IRQ_Handler?A

IRQ_Handler?A   PROC    CODE32
             
                SUB     LR, LR, #4              ; Update Link Register
                STMFD   SP!, {R0-R12, LR}       ; Save Workspace & LR into Stack
                MRS     R1, SPSR                ; Copy SPSR into R1
                STMFD   SP!, {R1}               ; Save SPSR into Stack
                LDR     LR, =ReturnAddress      ; Read Return Address
                LDR     R0, =EIC_BASE
                LDR     R1, =IVR_OFS
                ADD     PC, R0, R1              ; Branch to IRQ Handler
ReturnAddress
                ; Clear pending bit in EIC (using the proper IPRx)
                LDR     R0, =EIC_BASE
                LDR     R2, [R0, #CICR_OFS]     ; Get the IRQ Channel Number
                MOV     R3, #1
                MOV     R3, R3, LSL R2
                STR     R3, [R0, #IPR_OFS]      ; Clear corresponding IPR bit
                LDMFD   SP!, {R1}               ; Restore SPSR into R1
                MSR     SPSR_CXSF, R1           ; Restore SPSR
                LDMFD   SP!, {R0-R12, PC}^      ; Return to program
         
                ENDP


;*******************************************************************************
;                   Import IRQ handlers from 71x_it.c
;*******************************************************************************

EXTERN CODE32 (T0TIMI_IRQHandler?A)
EXTERN CODE32 (FLASH_IRQHandler?A)
EXTERN CODE32 (RCCU_IRQHandler?A)
EXTERN CODE32 (RTC_IRQHandler?A)
EXTERN CODE32 (WDG_IRQHandler?A)
EXTERN CODE32 (XTI_IRQHandler?A)
EXTERN CODE32 (USBHP_IRQHandler?A)
EXTERN CODE32 (I2C0ITERR_IRQHandler?A)
EXTERN CODE32 (I2C1ITERR_IRQHandler?A)
EXTERN CODE32 (UART0_IRQHandler?A)
EXTERN CODE32 (UART1_IRQHandler?A)
EXTERN CODE32 (UART2_IRQHandler?A)
EXTERN CODE32 (UART3_IRQHandler?A)
EXTERN CODE32 (BSPI0_IRQHandler?A)
EXTERN CODE32 (BSPI1_IRQHandler?A)
EXTERN CODE32 (I2C0_IRQHandler?A)
EXTERN CODE32 (I2C1_IRQHandler?A)
EXTERN CODE32 (CAN_IRQHandler?A)
EXTERN CODE32 (ADC12_IRQHandler?A)
EXTERN CODE32 (T1TIMI_IRQHandler?A)
EXTERN CODE32 (T2TIMI_IRQHandler?A)
EXTERN CODE32 (T3TIMI_IRQHandler?A)
EXTERN CODE32 (HDLC_IRQHandler?A)
EXTERN CODE32 (USBLP_IRQHandler?A)
EXTERN CODE32 (T0TOI_IRQHandler?A)
EXTERN CODE32 (T0OC1_IRQHandler?A)
EXTERN CODE32 (T0OC2_IRQHandler?A)


;*******************************************************************************
;* Macro Name     : IRQ_to_SYS
;* Description    : This macro used to switch form IRQ mode to SYS mode
;* Input          : none.
;* Output         : none
;*******************************************************************************
                    IRQ_to_SYS MACRO
                    MSR     CPSR_C, #0x1F       ; Switch to SYS Mode
                    STMFD   SP!, {LR}           ; Save Link Register
                    ENDM
;*******************************************************************************
;* Macro Name     : SYS_to_IRQ
;* Description    : This macro used to switch from SYS mode to IRQ mode
;                   then to return to IRQHnadler routine.
;* Input          : none.
;* Output         : none.
;*******************************************************************************
                    SYS_to_IRQ MACRO
                    LDMFD   SP!, {LR}           ; Restore Link Register 
                    MSR     CPSR_C, #0xD2       ; Switch to IRQ Mode
                    MOV     PC, LR              ; Return to IRQHandler routine to
                                                ; clear the pending bit
                    ENDM
;*******************************************************************************


;*******************************************************************************
;* Function Name  : T0TIMIIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T0TIMI_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T0TIMI_IRQHandler function termination.
;* Input          : none.
;* Output         : none.
;*******************************************************************************
PUBLIC              T0TIMIIRQHandler
T0TIMIIRQHandler    PROC CODE32
                    IRQ_to_SYS
                    BL  T0TIMI_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : FLASHIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the FLASH_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   FLASH_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              FLASHIRQHandler
FLASHIRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  FLASH_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : RCCUIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the RCCU_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   RCCU_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              RCCUIRQHandler
RCCUIRQHandler      PROC CODE32
                    IRQ_to_SYS
                    BL  RCCU_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : RTCIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the RTC_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   RTC_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              RTCIRQHandler
RTCIRQHandler       PROC CODE32
                    IRQ_to_SYS
                    BL  RTC_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : WDGIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the WDG_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   WDG_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              WDGIRQHandler
WDGIRQHandler       PROC CODE32
                    IRQ_to_SYS
                    BL  WDG_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : XTIIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the XTI_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   XTI_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              XTIIRQHandler
XTIIRQHandler       PROC CODE32
                    IRQ_to_SYS
                    BL  XTI_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : USBHPIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the USBHP_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   USBHP_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              USBHPIRQHandler
USBHPIRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  USBHP_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : I2C0ITERRIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the I2C0ITERR_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   I2C0ITERR_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              I2C0ITERRIRQHandler
I2C0ITERRIRQHandler PROC CODE32
                    IRQ_to_SYS
                    BL  I2C0ITERR_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : I2C1ITERRIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the I2C1ITERR_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   I2C1ITERR_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              I2C1ITERRIRQHandler
I2C1ITERRIRQHandler PROC CODE32
                    IRQ_to_SYS
                    BL  I2C1ITERR_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : UART0IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the UART0_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   UART0_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              UART0IRQHandler
UART0IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  UART0_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : UART1IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the UART1_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   UART1_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              UART1IRQHandler
UART1IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  UART1_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : UART2IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the UART2_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   UART2_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              UART2IRQHandler
UART2IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  UART2_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : UART3IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the UART3_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   UART3_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              UART3IRQHandler
UART3IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  UART3_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : BSPI0IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the BSPI0_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   BSPI0_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              BSPI0IRQHandler
BSPI0IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  BSPI0_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : BSPI1IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the BSPI1_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   BSPI1_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              BSPI1IRQHandler
BSPI1IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  BSPI1_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : I2C0IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the I2C0_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   I2C0_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              I2C0IRQHandler
I2C0IRQHandler      PROC CODE32
                    IRQ_to_SYS
                    BL  I2C0_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : I2C1IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the I2C1_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   I2C1_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              I2C1IRQHandler
I2C1IRQHandler      PROC CODE32
                    IRQ_to_SYS
                    BL  I2C1_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : CANIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the CAN_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   CAN_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              CANIRQHandler
CANIRQHandler       PROC CODE32
                    IRQ_to_SYS
                    BL  CAN_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : ADC12IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the ADC12_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   ADC12_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              ADC12IRQHandler
ADC12IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  ADC12_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : T1TIMIIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T1TIMI_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T1TIMI_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              T1TIMIIRQHandler
T1TIMIIRQHandler    PROC CODE32
                    IRQ_to_SYS
                    BL  T1TIMI_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : T2TIMIIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T2TIMI_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T2TIMI_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              T2TIMIIRQHandler
T2TIMIIRQHandler    PROC CODE32
                    IRQ_to_SYS
                    BL  T2TIMI_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : T3TIMIIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T3TIMI_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T3TIMI_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              T3TIMIIRQHandler
T3TIMIIRQHandler    PROC CODE32
                    IRQ_to_SYS
                    BL  T3TIMI_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : HDLCIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the HDLC_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   HDLC_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              HDLCIRQHandler
HDLCIRQHandler      PROC CODE32
                    IRQ_to_SYS
                    BL  HDLC_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : USBLPIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the USBLP_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   USBLP_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              USBLPIRQHandler
USBLPIRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  USBLP_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : T0TOIIRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T0TOI_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T0TOI_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              T0TOIIRQHandler
T0TOIIRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  T0TOI_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : T0OC1IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T0OC1_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T0OC1_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              T0OC1IRQHandler
T0OC1IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  T0OC1_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************
;* Function Name  : T0OC2IRQHandler
;* Description    : This function used to switch to SYS mode before entering
;                   the T0OC2_IRQHandler function located in 71x_it.c.
;                   Then to return to IRQ mode after the
;                   T0OC2_IRQHandler function termination.
;* Input          : none
;* Output         : none
;*******************************************************************************
PUBLIC              T0OC2IRQHandler
T0OC2IRQHandler     PROC CODE32
                    IRQ_to_SYS
                    BL  T0OC2_IRQHandler?A
                    SYS_to_IRQ
                    ENDP
;*******************************************************************************


                    END
